@@ -65,32 +65,27 @@ int *nx,*p,*antpers,*Ntimes,*sim,*retur,*antsim,*status,*id,*covariance,
  // {{{ setting up variables and allocating
   matrix *ldesignX,*wX,*A,*AI,*Vcov;
   matrix *cumAt[*antclust];
-  vector  *diag,*dB,*dN,*VdB,*xi,*rowX,*rowcum,*difX,*vtmp;
+  vector  *dB,*VdB,*xi,*rowX,*rowcum,*difX,*vtmp;
   vector *cumhatA[*antclust],*cumA[*antclust],*cum;
   int cin,ci=0,i,j,k,l,s,c,count,pers=0,
-      *cluster=calloc(*antpers,sizeof(int)),
-      *idd=calloc(*antpers,sizeof(int));
+      *cluster=calloc(*antpers,sizeof(int)), *idd=calloc(*antpers,sizeof(int));
 //  int *int0=calloc(*antpers,sizeof(int));
   double time,ahati,*vcudif=calloc((*Ntimes)*(*p+1),sizeof(double));
   double fabs(),sqrt();
-  
 
   if (*robust==1) {
-    for (i=0;i<*antclust;i++) { malloc_vec(*p,cumhatA[i]); 
-    malloc_vec(*p,cumA[i]); 
+    for (i=0;i<*antclust;i++) { malloc_vec(*p,cumhatA[i]); malloc_vec(*p,cumA[i]); 
     if (*sim==1) malloc_mat(*Ntimes,*p,cumAt[i]); } 
   }
-  malloc_vec(*p,cum); 
-  malloc_mat(*antpers,*p,ldesignX); malloc_mat(*antpers,*p,wX); 
-  malloc_mat(*p,*p,Vcov); malloc_mat(*p,*p,A); malloc_mat(*p,*p,AI);
-  malloc_vec(*antpers,dN); 
-  malloc_vec(*p,diag);malloc_vec(*p,dB);malloc_vec(*p,VdB);malloc_vec(*p,xi);
-  malloc_vec(*p,rowX); malloc_vec(*p,rowcum); malloc_vec(*p,difX); malloc_vec(*p,vtmp);
+  malloc_mats(*antpers,*p,&ldesignX,&wX,NULL); 
+  malloc_mats(*p,*p,&Vcov,&A,&AI,NULL);
+  malloc_vecs(*p,&cum,&dB,&VdB,&xi,&rowX,&rowcum,&difX,&vtmp,NULL);
 
   R_CheckUserInterrupt();
 
   for (c=0;c<*nx;c++) cluster[id[c]]=clusters[c]; 
   for (c=0;c<*nx;c++) idd[id[c]]=id[c]; 
+
   // }}}
 
 //     for (c=0;(c<*nx);c++) Rprintf(" %lf \n",weights[c]); 
@@ -143,7 +138,7 @@ for (s=1;s<*Ntimes;s++){
     invertS(A,AI,silent[0]); 
     if (ME(AI,0,0)==0.0 && *silent==0 && *strata==1){ 
        Rprintf(" X'X not invertible at time %lf \n",time); }
-    if (*strata==1)  {for (k=0;k<*p;k++) if (fabs(ME(A,k,k))<0.000001)  ME(AI,k,k)=0; else ME(AI,k,k)=1/ME(A,k,k);  }
+    if (*strata==1)  { for (k=0;k<*p;k++) if (fabs(ME(A,k,k))<0.000001)  ME(AI,k,k)=0; else ME(AI,k,k)=1/ME(A,k,k);  }
     if (s < -1) { print_mat(AI); print_mat(A);	}
 
     extract_row(wX,pers,xi);
@@ -217,15 +212,13 @@ for (s=1;s<*Ntimes;s++){
   }
 
   cu[0]=times[0]; vcu[0]=times[0]; robvcu[0]=times[0]; 
-  free_vec(xi); free_vec(rowX); free_vec(diag); free_vec(dB); free_vec(VdB);
-  free_vec(rowcum); free_vec(cum); free_vec(vtmp); free_mat(Vcov);
-  free_mat(ldesignX); free_mat(wX); 
+  free_vecs(&cum,&dB,&VdB,&xi,&rowX,&rowcum,&difX,&vtmp,NULL);
+  free_mats(&ldesignX,&wX,&Vcov,&A,&AI,NULL);
 
   if (*robust==1){
     for (i=0;i<*antclust;i++) {
       free_vec(cumA[i]); free_vec(cumhatA[i]); if (*sim==1) free_mat(cumAt[i]); } }
-  free(cluster); 
-  free(vcudif); 
+  free(cluster);  free(idd); free(vcudif); 
 } // }}}
 
 void semiaalen(alltimes,Nalltimes,Ntimes,designX,nx,px,designG,ng,pg,antpers,start,stop,nb,bhat,cu,vcu,Robvcu,gamma,Vgamma,RobVgamma,sim,antsim,test,testOBS,robust,status,Ut,simUt,id,weighted,cumAit,retur,covariance,covs,resample,gammaiid,Biid,clusters,antclust,intZHZ,intZHdN,deltaweight,silent,weights,entry,fixedgamma,mof,offsets)
@@ -244,40 +237,34 @@ int *nx,*px,*antpers,*Nalltimes,*Ntimes,*nb,*ng,*pg,*sim,*antsim,*robust,*status
   vector *VdB,*difX,*xi,*tmpv1,*tmpv2; 
   vector *dAoff,*dA,*rowX,*dN,*AIXWdN,*bhatt,*pbhat,*plamt;
   vector *korG,*pghat,*rowZ,*gam,*gamoff,*dgam,*ZHdN,*IZHdN,*zi,*offset;
-  int cin,ci=0,i,j,k,l,c,s,count,pers=0,pmax,stat,
+  int l1,cin,ci=0,i,j,k,l,c,s,count,pers=0,pmax,stat,
       *cluster=calloc(*antpers,sizeof(int)),
+      *stats=calloc(*Nalltimes,sizeof(int)), 
       *idd=calloc(*antpers,sizeof(int)),
-      *ls=calloc(*Ntimes,sizeof(int)); 
-  double time,dtime,fabs(),sqrt();
-  double ahati,ghati,hati;
+      *ls=calloc(*Ntimes,sizeof(int)),
+      detail=1; 
+  double time,dtime,fabs(),sqrt(),ahati,ghati,hati;
   double *vcudif=calloc((*Ntimes)*(*px+1),sizeof(double)),
 	 *times=calloc(*Ntimes,sizeof(double)),
          *cumoff=calloc((*Nalltimes)*(*px+1),sizeof(double)); 
 
-  malloc_mat(*antpers,*px,X); malloc_mat(*antpers,*px,WX); malloc_mat(*antpers,*pg,Z);
-  malloc_mat(*antpers,*pg,WZ); malloc_mat(*px,*px,Vcov); malloc_mat(*px,*px,A);
-  malloc_mat(*px,*px,AI); malloc_mat(*px,*px,GCdM1M2); malloc_mat(*pg,*pg,tmpM2);
-  malloc_mat(*pg,*pg,ZWZ); malloc_mat(*px,*px,VarKorG); malloc_mat(*pg,*pg,RobVargam);
-  malloc_mat(*pg,*pg,Vargam); malloc_mat(*pg,*pg,dVargam); malloc_mat(*pg,*pg,ICGam);
-  malloc_mat(*pg,*pg,CGam); malloc_mat(*pg,*pg,dCGam); malloc_mat(*px,*antpers,AIXW);
-  malloc_mat(*pg,*antpers,ZH);
+  malloc_mat(*antpers,*px,X); malloc_mat(*antpers,*px,WX); 
+  malloc_mat(*antpers,*pg,Z); malloc_mat(*antpers,*pg,WZ); 
+  malloc_mat(*px,*antpers,AIXW); malloc_mat(*pg,*antpers,ZH);
+  malloc_mats(*px,*px,&Vcov,&A,&AI,&GCdM1M2,&VarKorG,NULL); 
+  malloc_mats(*pg,*pg,&tmpM2,&ZWZ,&RobVargam,&Vargam,&dVargam,&ICGam,&CGam,&dCGam,NULL); 
   malloc_mats(*px,*pg,&tmpM3,&Ct,&dC,&XWZ,&XWZAI,&dM1M2,&M1M2t,NULL);
   malloc_mat(*px,*pg,tmpM4);
+
+  malloc_vecs(*px,&dA,&dAoff,&VdB,&difX,&xi,&tmpv1,&korG,&rowX,&AIXWdN,&bhatt,NULL);
+  malloc_vecs(*pg,&zi,&tmpv2,&rowZ,&gam,&gamoff,&dgam,&ZHdN,&IZHdN,NULL);
+  malloc_vecs(*antpers,&offset,&pbhat,&dN,&pghat,&plamt,NULL);
+
   for (j=0;j<*Nalltimes;j++) {
-	  malloc_mat(*px,*pg,Acorb[j]); malloc_mat(*px,*pg,C[j]);
+	  malloc_mat(*px,*pg,Acorb[j]); malloc_mat(*px,*pg,C[j]);stats[j]=0; 
   }
   for (j=0;j<*Ntimes;j++) { malloc_mat(*px,*pg,M1M2[j]); }
 
-  malloc_vec(*px,dA); malloc_vec(*px,dAoff); 
-  malloc_vec(*px,VdB); malloc_vec(*px,difX);
-  malloc_vec(*px,xi); malloc_vec(*px,tmpv1); malloc_vec(*px,korG);
-  malloc_vec(*px,rowX); malloc_vec(*px,AIXWdN); malloc_vec(*px,bhatt);
-  malloc_vec(*pg,zi); malloc_vec(*pg,tmpv2); malloc_vec(*pg,rowZ);
-  malloc_vec(*pg,gam); malloc_vec(*pg,gamoff); 
-  malloc_vec(*pg,dgam); malloc_vec(*pg,ZHdN);
-  malloc_vec(*pg,IZHdN);
-  malloc_vecs(*antpers,&offset,&pbhat,&dN,&pghat,&plamt,NULL);
-
   if (*robust==1) {
 	  for (j=0;j<*antclust;j++) { malloc_mat(*Ntimes,*px,W3t[j]);
 		  malloc_mat(*Ntimes,*px,W4t[j]); malloc_vec(*pg,W2[j]); 
@@ -319,7 +306,7 @@ int *nx,*px,*antpers,*Nalltimes,*Ntimes,*nb,*ng,*pg,*sim,*antsim,*robust,*status
 					  if ((j<*px) & (k<*pg)) ME(XWZ,j,k)+=ME(X,id[c],j)*ME(Z,id[c],k)*weights[c]; 
 					  if ((j<*pg) & (k<*pg)) ME(ZWZ,j,k)+= ME(Z,id[c],j)*ME(Z,id[c],k)*weights[c]; 
 				  }
-				  if (time==stop[c] && status[c]==1) { pers=id[c];stat=1;l=l+1; ls[l]=s; }
+				  if (time==stop[c] && status[c]==1) { pers=id[c];stat=1;l=l+1; ls[l]=s;stats[s]=stat; }
 				  count=count+1; 
 			  }
 		  }
@@ -363,21 +350,13 @@ int *nx,*px,*antpers,*Nalltimes,*Ntimes,*nb,*ng,*pg,*sim,*antsim,*robust,*status
 
 	  // Rprintf("ci ci ci  %lf %lf %ld \n",time,stop[ci],status[ci]); 
 	  if ((s>1) & (time==stop[ci]) & (status[ci]==1)) {
-		  pers=id[ci]; stat=1;l=l+1; ls[l]=s;
+		  pers=id[ci]; stat=1;l=l+1; ls[l]=s;stats[s]=stat; 
 	  }
 
 	  // }}}
 
 	  // }}}
 
-	  //   Rprintf(" hej mig \n"); 
-	  // Rprintf("lllllllllll %lf %lf %d %d %d \n",time,dtime,pers,stat,l); 
-	  //   print_mat(X); print_mat(Z); 
-	  //   print_mat(A); print_mat(ZWZ); print_mat(XWZ); 
-	  //   Rprintf("========================================== \n"); 
-	  //readXZt(antpers,nx,px,designX,pg,designG,start,stop,status,pers,X,WX,Z,WZ,time,
-	  //	clusters,cluster,ls,stat,l,id,s,1);
-
 	  invertS(A,AI,silent[0]);
 	  if (ME(AI,0,0)==0.0 && *silent==0){ 
 		  Rprintf(" X'X not invertible at time %lf \n",time);
@@ -411,7 +390,6 @@ int *nx,*px,*antpers,*Nalltimes,*Ntimes,*nb,*ng,*pg,*sim,*antsim,*robust,*status
 		  //	print_vec(tmpv1); 
 	  } 
 
-
 	  scl_mat_mult(dtime,XWZAI,tmpM4);
 	  mat_add(tmpM4,Ct,Ct); mat_copy(Ct,C[s]);
 
@@ -439,25 +417,24 @@ int *nx,*px,*antpers,*Nalltimes,*Ntimes,*nb,*ng,*pg,*sim,*antsim,*robust,*status
 
     } /* s =1...Ntimes */ 
 
+ if (detail>=2) Rprintf("Fitting done \n"); 
   invertS(CGam,ICGam,silent[0]);
   if (*fixedgamma==0) Mv(ICGam,IZHdN,gam); 
   if ((*mof==1) & (*fixedgamma==0)) {
       Mv(ICGam,gamoff,dgam); vec_subtr(gam,dgam,gam);
   }
-
  
   if (ME(ICGam,0,0)==0 && *silent==0) Rprintf(" intZHZ  singular\n"); 
 //  Mv(ICGam,IZHdN,gam); 
   MxA(Vargam,ICGam,tmpM2); 
   MxA(ICGam,tmpM2,Vargam); 
 
-
   mat_zeros(X); mat_zeros(Z); mat_zeros(WX); mat_zeros(WZ);
-  l=0; 
+  l=0; l1=0; 
   for (s=1;s<*Nalltimes;s++) {
-    time=alltimes[s]; vec_zeros(dN); dtime=time-alltimes[s-1]; 
-    stat=0; 
+    time=alltimes[s]; dtime=time-alltimes[s-1]; stat=0; 
 
+    if (*robust==1) { 
   // {{{ reading design and making matrix products
    if (s==1)  { // {{{ reading start design 
      for (c=0,count=0;((c<*nx) && (count!=*antpers));c++) {
@@ -470,7 +447,8 @@ int *nx,*px,*antpers,*Nalltimes,*Ntimes,*nb,*ng,*pg,*sim,*antsim,*robust,*status
 	    if (j<*pg) { ME(WZ,id[c],j)=weights[c]*designG[j*(*ng)+c]; } 
 	  }
 	  if (time==stop[c] && status[c]==1) {
-	    pers=id[c];stat=1;l=l+1; ls[l]=s;
+	    pers=id[c];stat=1; l1=l1+1; 
+//	    ls[l1]=s;
 	  }
 	  count=count+1; 
 	}
@@ -480,18 +458,11 @@ int *nx,*px,*antpers,*Nalltimes,*Ntimes,*nb,*ng,*pg,*sim,*antsim,*robust,*status
     while ((stop[ci]<time)  & (ci>=0) )  ci=ci-1; 
     } // }}}
 
-//   print_mat(X); print_mat(Z); 
-//   print_mat(WX); print_mat(WZ); 
-// Rprintf(" (((((((((((((((((((((((((((((((((((((((((((( \n"); 
-// Rprintf("%d %d %d %lf %lf %lf \n",s,ci,id[ci],start[ci],stop[ci],time); 
-
      vec_zeros(rowX); vec_zeros(rowZ); 
     if (s>1)  // {{{ modifying design for next time points
     while ((stop[ci]<time)  & (ci>=0) ) {
-// Rprintf("ww %d %d  %lf %lf %d \n",ci,id[ci],stop[ci],time,entry[ci]); 
             for(j=0;j<*px;j++) VE(xi,j)=designX[j*(*nx)+ci]; 
             for(j=0;j<*pg;j++) VE(zi,j)=designG[j*(*nx)+ci]; 
-//            print_vec(xi); print_vec(zi); 
 	    if (entry[ci]==1)  {
                 VE(offset,id[ci])=offsets[ci]; 
 	        replace_row(X,id[ci],xi); replace_row(Z,id[ci],zi); 
@@ -517,27 +488,27 @@ int *nx,*px,*antpers,*Nalltimes,*Ntimes,*nb,*ng,*pg,*sim,*antsim,*robust,*status
 	  ci=ci-1; 
     }
 
-// Rprintf("ci ci ci  %lf %lf %ld \n",time,stop[ci],status[ci]); 
     if ((s>1) & (time==stop[ci]) & (status[ci]==1)) {
-         pers=id[ci]; stat=1;l=l+1; ls[l]=s;
+         pers=id[ci]; stat=1; l1=l1+1; 
+//       ls[l]=s;
+    }
+    // }}}
+    // }}}
     }
 
-    // }}}
-    
-    // }}}
+    if (stats[s]==1) l=l+1; 
+    if (stats[s]==1) for (k=1;k<=*px;k++) VE(dA,k-1)=cu[k*(*Ntimes)+l]; else vec_zeros(dA);
 
-    if (stat==1) for (k=1;k<=*px;k++) VE(dA,k-1)=cu[k*(*Ntimes)+l]; 
-    else vec_zeros(dA);
-    
+//    Rprintf(" %d %d %d %d %d %d \n",s,l,l1,ls[l],stats[s],stat);
 
-    if (*mof==1) for (k=1;k<=*px;k++) {
-	      VE(dA,k-1)=VE(dA,k-1)-cumoff[k*(*Nalltimes)+s];
-              VE(dAoff,k-1)=VE(dAoff,k-1)-cumoff[k*(*Nalltimes)+s];
-              cumoff[k*(*Nalltimes)+s]=cumoff[k*(*Nalltimes)+s-1]+
-		                       cumoff[k*(*Nalltimes)+s];
+    if (*mof==1) {
+          for (k=1;k<=*px;k++) {
+	  VE(dA,k-1)=VE(dA,k-1)-cumoff[k*(*Nalltimes)+s];
+          VE(dAoff,k-1)= -cumoff[k*(*Nalltimes)+s];
+          cumoff[k*(*Nalltimes)+s]=cumoff[k*(*Nalltimes)+s-1]+cumoff[k*(*Nalltimes)+s];
+          }
     }
 
-
     /* terms for robust variance   */ 
     if (*robust==1 )  // {{{
     { 
@@ -580,38 +551,30 @@ int *nx,*px,*antpers,*Nalltimes,*Ntimes,*nb,*ng,*pg,*sim,*antsim,*robust,*status
 	} /* i=1..antpers */ 
     } // }}} /* robust ==1 */
 
-    if (stat==1) {
+    if (stats[s]==1) {
 //      extract_row(X,pers,xi); ahati=vec_prod(xi,dA); 
-//      extract_row(WX,pers,xi);
-//      vec_star(xi,dA,rowX); 
+//      extract_row(WX,pers,xi); vec_star(xi,dA,rowX); 
 
-      for (k=1;k<=*px;k++) 
-	      cu[k*(*Ntimes)+l]=cu[k*(*Ntimes)+l-1]+cu[k*(*Ntimes)+l]+VE(dAoff,k-1); 
-     vec_zeros(dAoff); 
+      for (k=1;k<=*px;k++) cu[k*(*Ntimes)+l]=cu[k*(*Ntimes)+l-1]+cu[k*(*Ntimes)+l]+VE(dAoff,k-1); 
 
       MxA(C[ls[l]],Vargam,tmpM4); 
       MAt(tmpM4,C[ls[l]],VarKorG);
       MxA(M1M2[l],ICGam,tmpM4); 
       MAt(C[ls[l]],tmpM4,GCdM1M2);
 
-      for (k=1;k<=*px;k++) {
-	vcu[k*(*Ntimes)+l]= vcu[k*(*Ntimes)+l]+ME(VarKorG,k-1,k-1)-2.0*ME(GCdM1M2,k-1,k-1); 
-      } 
+     for (k=1;k<=*px;k++) vcu[k*(*Ntimes)+l]+= ME(VarKorG,k-1,k-1)-2.0*ME(GCdM1M2,k-1,k-1); 
     }
-
-
   } /* s=1 ..Ntimes */ 
 
   vec_star(IZHdN,gam,rowZ);    
 
+ if (detail>=2) Rprintf("Offsets adjustment and MG variances for gamma \n"); 
 //  loglike[0]=loglike[0]-vec_sum(rowZ); 
 
   for (s=1;s<*Ntimes;s++) {
 
     Mv(C[ls[s]],gam,korG); 
-    for (k=1;k<=*px;k++) {
-      cu[k*(*Ntimes)+s]=cu[k*(*Ntimes)+s]-VE(korG,k-1); 
-    }
+    for (k=1;k<=*px;k++) cu[k*(*Ntimes)+s]=cu[k*(*Ntimes)+s]-VE(korG,k-1); 
 
     if (*robust==1) { // {{{ ROBUST VARIANCES  
       vec_zeros(VdB); mat_zeros(Vcov); 
@@ -653,6 +616,7 @@ int *nx,*px,*antpers,*Nalltimes,*Ntimes,*nb,*ng,*pg,*sim,*antsim,*robust,*status
     }
   }  /*  s=1 ..Ntimes */ 
 
+ if (detail>=2) Rprintf("Baseline corrected for gamma and robust variances \n"); 
 
   if (*robust==1) {
     MxA(RobVargam,ICGam,tmpM2); 
@@ -676,27 +640,25 @@ int *nx,*px,*antpers,*Nalltimes,*Ntimes,*nb,*ng,*pg,*sim,*antsim,*robust,*status
   }
 
   // {{{ freeing
-  free_mat(Vcov); free_mat(X); free_mat(WX); free_mat(Z); free_mat(WZ); free_mat(A); 
-  free_mat(AI); free_mat(ZWZ); free_mat(VarKorG);free_mat(Vargam);free_mat(dVargam); 
-  free_mat(ICGam); free_mat(CGam); free_mat(dCGam); free_mat(M1M2t); 
-  free_mat(dM1M2); free_mat(Ct); free_mat(AIXW); free_mat(ZH); 
-  free_mat(dC); free_mat(XWZ); free_mat(XWZAI); free_mat(GCdM1M2); 
-  free_mat(tmpM2); free_mat(tmpM3); free_mat(tmpM4);
-  free_vec(dA); free_vec(tmpv1); free_vec(tmpv2); free_vec(difX); 
-  free_vec(korG); free_vec(rowX); free_vec(AIXWdN); free_vec(bhatt); free_vec(zi); 
-  free_vec(rowZ); free_vec(gam); free_vec(dgam); free_vec(ZHdN); free_vec(IZHdN); 
-  free_vec(dAoff); free_vec(gamoff); 
-  free_vec(dN); free_vec(pbhat); free_vec(pghat); free_vec(plamt); 
-  free_vec(offset); 
-  for (j=0;j<*Ntimes;j++) { free_mat(M1M2[j]);  }
+  
+  free_mat(X); free_mat(WX); free_mat(Z); free_mat(WZ); free_mat(AIXW); free_mat(ZH);
+  free_mats(&Vcov,&A,&AI,&GCdM1M2,&VarKorG,NULL); 
+  free_mats(&tmpM2,&ZWZ,&RobVargam,&Vargam,&dVargam,&ICGam,&CGam,&dCGam,NULL); 
+  free_mats(&tmpM4,&tmpM3,&Ct,&dC,&XWZ,&XWZAI,&dM1M2,&M1M2t,NULL);
+
+  free_vecs(&dA,&dAoff,&VdB,&difX,&xi,&tmpv1,&korG,&rowX,&AIXWdN,&bhatt,NULL);
+  free_vecs(&zi,&tmpv2,&rowZ,&gam,&gamoff,&dgam,&ZHdN,&IZHdN,NULL);
+  free_vecs(&offset,&pbhat,&dN,&pghat,&plamt,NULL);
+
   for (j=0;j<*Nalltimes;j++) { free_mat(Acorb[j]); free_mat(C[j]); }
+  for (j=0;j<*Ntimes;j++) { free_mat(M1M2[j]);  }
   if (*robust==1) {
     for (j=0;j<*Nalltimes;j++) free_mat(AIs[j]); 
     for (j=0;j<*antclust;j++) { 
       free_mat(W3t[j]); free_mat(W4t[j]); free_vec(W2[j]); free_vec(W3[j]);    } 
   }
-  free(ls); free(vcudif); free(times); free(cluster); 
-  free(cumoff); 
+  free(vcudif); free(times); free(cumoff); 
+  free(stats); free(idd); free(cluster); free(ls); 
   // }}}
 } // }}}
 
